#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <getopt.h>
#include <unistd.h>

#include "libfleximon.h"

#define CMDLINE_OPTIONS "p:d:htf"

int main (int argc, char **argv)
{
  int opt;
  int port = FM_DEFAULT_PORT;
  int dirs = 0;
  int testing = 0;
  int forwarding = 0;
  opt = getopt(argc,argv,CMDLINE_OPTIONS);
  while (opt > -1) {
    switch (opt) {
    case 'p': 
      port = atoi(optarg);
      if (port < 1024) {
	printf("Ports < 1024 are not allowed. Ignoring -p %i.\n",port);
	port = 9000;
      }
      printf("Set port to: %i\n",port);
      break;
    case 'd':
      printf("Loading plugins from %s ...\n",optarg);
      fm_load_plugins(optarg);
      dirs++;
      break;
    case 't':
      testing = 1;
      printf("Test mode activated (won't become daemon).\n");
      break;
    case 'f':
      forwarding = 1;
      printf("Forwarding (proxy capability) enabled.\n");
      break;
    case 'h':
      printf("Usage: %s [-p port] [-f] [-t] -d plugindir [-d plugindir2...]\n"
	     "Default port number is 9000. You must specify at least one "
	     "plugin directory. -t is test mode, ie. don't become a daemon.\n"
	     "-f enables forwarding of requests.",
	     argv[0]);
      return 0;
    default:
      printf("Illegal option -%c. Try %s -h for help.\n",opt,argv[0]);
      return 1;
    }
    opt = getopt(argc,argv,CMDLINE_OPTIONS);
  }
  if (!dirs) {
    printf("No plugin directories specified. Use -d and give at least one\n"
	   "plugin directory. Try %s -h for help.\n", argv[0]);
    return 1;
  } else {
    printf("Plugins loaded from %i directories.\n",dirs);
  }
  if (!testing) {
    if (fork() != 0) return 0;
  }
  fm_run_server(port,forwarding);
  return 0;
}
