#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <regex.h>

#include "libfleximon.h"


#define MAX_BUFFER_SIZE 1600
#define MAX_MATCHES 16


/* Define board
 *
 * Currently these boards are supported:
 * - ASUS_A7S_VM
 * - GIGABYTE_7IXE4
 */

#define ASUS_A7S_VM


/* Figure out regexes for the various boards */

#if defined(ASUS_A7S_VM)
/* Regexes for ASUS A7S-VM board (EKPplus nodes) */
#define CPU_TEMP_REGEXP "^CPU Temp:( *)\\+(.*)C "
#define MB_TEMP_REGEXP  "^MB Temp:( *)\\+(.*)C "
#define CPU_FAN_REGEXP  "^CPU Fan:( *)(.*) RPM "
#define MB_FAN_REGEXP   "^Ext. Fan:( *)(.*) RPM "

#elif defined(GIGABYTE_7IXE4)
/* Regexes for Gigabyte GA-7IXE4 board (many desktops) */
#define CPU_TEMP_REGEXP "^temp2:( *)\\+(.*)C "
#define MB_TEMP_REGEXP  "^temp1:( *)\\+(.*)C "
#define CPU_FAN_REGEXP  "^fan2:( *)(.*) RPM "
#define MB_FAN_REGEXP   "^fan1:( *)(.*) RPM "

#else
/* Unknown board, will never match */
#define CPU_TEMP_REGEXP "unmatched"
#define MB_TEMP_REGEXP  "unmatched"
#define CPU_FAN_REGEXP  "unmatched"
#define MB_FAN_REGEXP   "unmatched"

#endif


/* extract match substring from source string into dest */
static void extract (char *dest, const char *src, regmatch_t m)
{
  int i;
  dest[0] = 0;
  if (m.rm_so < 0) return;
  for (i=m.rm_so; i<m.rm_eo; i++)
    dest[i-m.rm_so] = src[i];
  dest[m.rm_eo-m.rm_so] = 0;
}


/* responder function for sensors command */
static char *respond_sensors (char *query)
{
  static int regi = 0;
  static regex_t cpu_temp_rex;
  static regex_t mb_temp_rex;
  static regex_t cpu_fan_rex;
  static regex_t mb_fan_rex;
  static char rebuffer [MAX_BUFFER_SIZE];
  FILE *file;
  char buffer [MAX_BUFFER_SIZE];
  char match [MAX_BUFFER_SIZE];
  regmatch_t matches [MAX_MATCHES];

  /* buffer initialisation */ 
  memset(buffer,0,MAX_BUFFER_SIZE);
  memset(rebuffer,0,MAX_BUFFER_SIZE);

  /* call sensors */
  system("sensors >test.dat");

  /* open file and read buffer */
  file = fopen("test.dat","r");
  if (!file) {
    return "No sensors available.\n";
  }

  /* compile regular expressions */
  if (!regi) {
    regcomp(&cpu_temp_rex,CPU_TEMP_REGEXP,REG_EXTENDED);
    regcomp(&mb_temp_rex,MB_TEMP_REGEXP,REG_EXTENDED);
    regcomp(&cpu_fan_rex,CPU_FAN_REGEXP,REG_EXTENDED);
    regcomp(&mb_fan_rex,MB_FAN_REGEXP,REG_EXTENDED);
    regi = 1;
  }

  /* line-by-line scan through file */
  while (fgets(buffer,MAX_BUFFER_SIZE,file)) {
    if (regexec(&cpu_temp_rex,buffer,MAX_MATCHES,matches,0) ==0 ) {
      /* cpu temp match */
      extract(match,buffer,matches[2]);
      strcat(rebuffer,"CPU temperature: ");
      strcat(rebuffer,match);
      strcat(rebuffer,"\n");
    } else if (regexec(&mb_temp_rex,buffer,MAX_MATCHES,matches,0) == 0) {
      /* mobo temp match */
      extract(match,buffer,matches[2]);
      strcat(rebuffer,"Mobo temperature: ");
      strcat(rebuffer,match);
      strcat(rebuffer,"\n");
    } else if (regexec(&cpu_fan_rex,buffer,MAX_MATCHES,matches,0) == 0) {
      /* cpu fan match */
      extract(match,buffer,matches[2]);
      strcat(rebuffer,"CPU fan: ");
      strcat(rebuffer,match);
      strcat(rebuffer,"\n");
    } else if (regexec(&mb_fan_rex,buffer,MAX_MATCHES,matches,0) == 0) {
      /* mobo fan match */
      extract(match,buffer,matches[2]);
      strcat(rebuffer,"Mobo fan: ");
      strcat(rebuffer,match);
      strcat(rebuffer,"\n");
    }
  }

  /* close file and return */
  fclose(file);
  return rebuffer;
}

 
FM_PLUGIN("SENSORS","Mainboard sensors",respond_sensors)
